@@ -1,10 +1,10 @@
 #ifndef CHECKMATE_ALL_NCHAR_H_
 #define CHECKMATE_ALL_NCHAR_H_
 
-#include <R.h>
 #define USE_RINTERNALS
+#include <R.h>
 #include <Rinternals.h>
 
-Rboolean all_nchar(SEXP, R_xlen_t);
+Rboolean all_nchar(SEXP, R_xlen_t, Rboolean);
 
 #endif
