@@ -1,12 +1,13 @@
 #ifndef CHECKMATE_IS_INTEGERISH_H_
 #define CHECKMATE_IS_INTEGERISH_H_
 
-#include <R.h>
 #define USE_RINTERNALS
+#include <R.h>
 #include <Rinternals.h>
-#define INTEGERISH_DEFAULT_TOL sqrt(DOUBLE_EPS)
+#include <R_ext/Visibility.h>
 
-Rboolean isIntegerish(SEXP, double);
-SEXP c_is_integerish(SEXP, SEXP);
+#define INTEGERISH_DEFAULT_TOL sqrt(DOUBLE_EPS)
+Rboolean isIntegerish(SEXP, double, Rboolean);
+SEXP attribute_hidden c_is_integerish(SEXP, SEXP);
 
 #endif
