@@ -1,17 +1,24 @@
 #include "qassert.h"
 #include "helper.h"
+#include "guess_type.h"
 #include "any_missing.h"
 #include "is_integerish.h"
-#include "cmessages.h"
 
 typedef enum {
-    CL_LOGICAL, CL_INTEGER, CL_INTEGERISH, CL_NUMERIC, CL_DOUBLE, CL_STRING, CL_LIST, CL_COMPLEX,
-    CL_ATOMIC, CL_ATOMIC_VECTOR, CL_MATRIX, CL_DATAFRAME, CL_ENVIRONMENT, CL_FUNCTION, CL_NULL, CL_NONE
+    CL_LOGICAL, CL_INTEGER, CL_INTEGERISH, CL_NUMERIC, CL_DOUBLE, CL_STRING, CL_FACTOR, CL_LIST, CL_COMPLEX,
+    CL_ATOMIC, CL_ATOMIC_VECTOR, CL_MATRIX, CL_DATAFRAME, CL_FUNCTION, CL_ENVIRONMENT, CL_NULL, CL_NONE
 } class_t;
 
+static const char * CLSTR[] = {
+     "logical", "integer", "integerish", "numeric", "double", "string", "factor", "list", "complex",
+     "atomic", "atomic vector", "matrix", "data frame", "function", "environment", "NULL"
+};
+
+typedef enum { LT, LE, EQ, GE, GT, NE, NONE } cmp_t;
+static const char * CMPSTR[] = { "<", "<=", "==", ">=", ">", "!=" };
+
 typedef Rboolean(*dd_cmp)(double, double);
 typedef Rboolean(*ll_cmp)(R_xlen_t, R_xlen_t);
-typedef enum { LT, LE, EQ, GE, GT, NE, NONE } cmp_t;
 typedef struct { dd_cmp fun; double cmp; cmp_t op; } bound_t;
 
 typedef struct {
@@ -31,6 +38,12 @@ typedef struct {
     bound_t upper;
 } checker_t;
 
+typedef struct {
+    Rboolean ok;
+    char msg[255];
+} msg_t;
+
+
 static inline Rboolean ii_eq(const R_xlen_t x, const R_xlen_t y) { return x == y; }
 static inline Rboolean ii_lt(const R_xlen_t x, const R_xlen_t y) { return x <  y; }
 static inline Rboolean ii_gt(const R_xlen_t x, const R_xlen_t y) { return x >  y; }
@@ -43,25 +56,31 @@ static inline Rboolean dd_ge(const double x, const double y) { return x >= y; }
 static inline Rboolean dd_ne(const double x, const double y) { return x != y; }
 static inline Rboolean is_class_logical(SEXP x) { return isLogical(x); }
 static inline Rboolean is_class_integer(SEXP x) { return isInteger(x); }
-static inline Rboolean is_class_integerish(SEXP x) { return isIntegerish(x, INTEGERISH_DEFAULT_TOL); }
+static inline Rboolean is_class_integerish(SEXP x) { return isIntegerish(x, INTEGERISH_DEFAULT_TOL, TRUE); }
 static inline Rboolean is_class_double(SEXP x) { return isReal(x); }
 static inline Rboolean is_class_numeric(SEXP x) { return isStrictlyNumeric(x); }
 static inline Rboolean is_class_complex(SEXP x) { return isComplex(x); }
 static inline Rboolean is_class_string(SEXP x) { return isString(x); }
+static inline Rboolean is_class_factor(SEXP x) { return isFactor(x); }
 static inline Rboolean is_class_atomic(SEXP x) { return isNull(x) || isVectorAtomic(x); }
-static inline Rboolean is_class_atomic_vector(SEXP x) { return isVectorAtomic(x); }
+static inline Rboolean is_class_atomic_vector(SEXP x) { return isAtomicVector(x); }
 static inline Rboolean is_class_list(SEXP x) { return isRList(x); }
 static inline Rboolean is_class_matrix(SEXP x) { return isMatrix(x); }
 static inline Rboolean is_class_frame(SEXP x) { return isFrame(x); }
-static inline Rboolean is_class_function(SEXP x) { return isFunction(x); }
 static inline Rboolean is_class_environment(SEXP x) { return isEnvironment(x); }
 static inline Rboolean is_class_null(SEXP x) { return isNull(x); }
 
-static const char * CMPSTR[] = { "<", "<=", "==", ">=", ">", "!=" };
-static const char * CLSTR[] = {
-     "logical", "integer", "integerish", "numeric", "double", "string", "list", "complex",
-     "atomic", "atomic vector", "matrix", "data frame", "environment", "function", "NULL"
-};
+static const msg_t MSGT = { .ok = TRUE };
+static const msg_t MSGF = { .ok = FALSE };
+
+static msg_t message(const char *fmt, ...) {
+    msg_t msg = { .ok = FALSE };
+    va_list vargs;
+    va_start(vargs, fmt);
+    vsnprintf(msg.msg, 255, fmt, vargs);
+    va_end(vargs);
+    return msg;
+}
 
 /*********************************************************************************************************************/
 /* Some helper functions                                                                                             */
@@ -71,18 +90,33 @@ static msg_t check_bound(SEXP x, const bound_t bound) {
         const double *xp = REAL(x);
         const double * const xend = xp + xlength(x);
         for (; xp != xend; xp++) {
-            if (!ISNAN(*xp) && !bound.fun(*xp, bound.cmp))
-                return make_msg("All elements must be %s %g", CMPSTR[bound.op], bound.cmp);
+            if (!ISNAN(*xp) && !bound.fun(*xp, bound.cmp)) {
+                if (bound.cmp == R_PosInf)
+                    return message("All elements must be %s Inf", CMPSTR[bound.op]);
+                if (bound.cmp == R_NegInf)
+                    return message("All elements must be %s -Inf", CMPSTR[bound.op]);
+                return message("All elements must be %s %g", CMPSTR[bound.op], bound.cmp);
+            }
         }
     } else if (isInteger(x)) {
         const int *xp = INTEGER(x);
         const int * const xend = xp + xlength(x);
         for (; xp != xend; xp++) {
             if (*xp != NA_INTEGER && !bound.fun((double) *xp, bound.cmp))
-                return make_msg("All elements must be %s %g", CMPSTR[bound.op], bound.cmp);
+                return message("All elements must be %s %g", CMPSTR[bound.op], bound.cmp);
         }
+    } else if (isString(x)) {
+        const R_xlen_t nx = xlength(x);
+        double nchar;
+        for (R_xlen_t i = 0; i < nx; i++) {
+            nchar = STRING_ELT(x, i) == NA_STRING ? 0. : (double)length(STRING_ELT(x, i));
+            if (!bound.fun(nchar, bound.cmp))
+                return message("All elements must have %s %g chars", CMPSTR[bound.op], bound.cmp);
+        }
+    } else if (isFactor(x)) {
+        return check_bound(getAttrib(x, R_LevelsSymbol), bound);
     } else {
-        error("Bound checks only possible for numeric variables");
+        error("Bound checks only possible for numeric variables, strings and factors, not %s", guess_type(x));
     }
 
     return MSGT;
@@ -130,6 +164,12 @@ static int parse_class(checker_t *checker, const char *rule) {
             checker->class.fun = &is_class_string;
             checker->class.name = CL_STRING;
             break;
+        case 'F':
+            checker->missing.fun = &any_missing_integer;
+        case 'f':
+            checker->class.fun = &is_class_factor;
+            checker->class.name = CL_FACTOR;
+            break;
         case 'L':
             checker->missing.fun = &any_missing_list;
         case 'l':
@@ -170,10 +210,6 @@ static int parse_class(checker_t *checker, const char *rule) {
             checker->class.fun = &is_class_environment;
             checker->class.name = CL_ENVIRONMENT;
             break;
-        case 'f':
-            checker->class.fun = &is_class_function;
-            checker->class.name = CL_FUNCTION;
-            break;
         case '0':
             checker->class.fun = &is_class_null;
             checker->class.name = CL_NULL;
@@ -284,6 +320,7 @@ static int parse_bounds(checker_t *checker, const char *rule) {
         if (checker->lower.op == GT) {
             checker->lower.fun = &dd_ne;
             checker->lower.cmp = R_NegInf;
+            checker->lower.op  = NE;
         } else {
             checker->lower.fun = NULL;
         }
@@ -300,13 +337,14 @@ static int parse_bounds(checker_t *checker, const char *rule) {
 
     cmp = strtod(start, &end);
     if (*end == ')') {
-        checker->upper.op = LT;
         if (start == end) {
             checker->upper.fun = &dd_ne;
             checker->upper.cmp = R_PosInf;
+            checker->upper.op = NE;
         } else {
             checker->upper.fun = &dd_lt;
             checker->upper.cmp = cmp;
+            checker->upper.op = LT;
         }
     } else if (*end == ']') {
         if (start == end) {
@@ -314,6 +352,7 @@ static int parse_bounds(checker_t *checker, const char *rule) {
         } else {
             checker->upper.fun = &dd_le;
             checker->upper.cmp = cmp;
+            checker->upper.op = LE;
         }
     } else {
         error("Invalid bound definition, error parsing upper bound or missing closing ')' or ']': %s", rule);
@@ -332,7 +371,7 @@ static void parse_rule(checker_t *checker, const char *rule) {
     rule += parse_bounds(checker, rule);
     if (rule[0] == '\0')
         return;
-    error("Additional chars found!");
+    error("Additional chars found in rule!");
 }
 
 /*********************************************************************************************************************/
@@ -340,15 +379,15 @@ static void parse_rule(checker_t *checker, const char *rule) {
 /*********************************************************************************************************************/
 static msg_t check_rule(SEXP x, const checker_t *checker, const Rboolean err_msg) {
     if (checker->class.fun != NULL && !checker->class.fun(x)) {
-        return err_msg ? make_msg("Must be of class '%s', not '%s'", CLSTR[checker->class.name], guessType(x)) : MSGF;
+        return err_msg ? message("Must be of class '%s', not '%s'", CLSTR[checker->class.name], guess_type(x)) : MSGF;
     }
 
     if (checker->missing.fun != NULL && checker->missing.fun(x)) {
-        return err_msg ? make_msg("May not contain missing values") : MSGF;
+        return err_msg ? message("May not contain missing values") : MSGF;
     }
 
     if (checker->len.fun != NULL && !checker->len.fun(xlength(x), checker->len.cmp)) {
-        return err_msg ? make_msg("Must be of length %s %i, but has length %g", CMPSTR[checker->len.op], checker->len.cmp, (double)xlength(x)) : MSGF;
+        return err_msg ? message("Must be of length %s %i, but has length %g", CMPSTR[checker->len.op], checker->len.cmp, (double)xlength(x)) : MSGF;
     }
 
     if (checker->lower.fun != NULL) {
@@ -390,7 +429,17 @@ static inline R_len_t qassert_list(SEXP x, const checker_t *checker, msg_t *resu
     return 0;
 }
 
-SEXP c_qassert(SEXP x, SEXP rules, SEXP recursive) {
+/* exported for other packages */
+SEXP qassert(SEXP x, const char *rule, const char *name) {
+    checker_t checker;
+    parse_rule(&checker, rule);
+    msg_t result = check_rule(x, &checker, TRUE);
+    if (!result.ok)
+        error("Variable '%s': %s", name, result.msg);
+    return x;
+}
+
+SEXP attribute_hidden c_qassert(SEXP x, SEXP rules, SEXP recursive) {
     const Rboolean nrules = length(rules);
     R_len_t failed;
     if (!isString(rules))
@@ -414,6 +463,7 @@ SEXP c_qassert(SEXP x, SEXP rules, SEXP recursive) {
     } else {
         failed = qassert1(x, checker, result, nrules);
     }
+
     if (failed == 0)
         return ScalarLogical(TRUE);
 
@@ -439,19 +489,38 @@ static inline Rboolean qtest1(SEXP x, const checker_t *checker, const R_len_t nr
     return FALSE;
 }
 
-static inline Rboolean qtest_list(SEXP x, const checker_t *checker, const R_len_t nrules) {
-    if (!isNewList(x))
+static inline Rboolean qtest_list(SEXP x, const checker_t *checker, const R_len_t nrules, R_len_t depth) {
+    if (!isNewList(x) || isNull(x))
         error("Argument 'x' must be a list or data.frame");
 
     const R_len_t nx = xlength(x);
-    for (R_xlen_t i = 0; i < nx; i++) {
-        if (!qtest1(VECTOR_ELT(x, i), checker, nrules))
-            return FALSE;
+    if (depth > 1) {
+        for (R_xlen_t i = 0; i < nx; i++) {
+            if (isRList(VECTOR_ELT(x, i))) {
+                if (!qtest_list(VECTOR_ELT(x, i), checker, nrules, depth - 1))
+                    return FALSE;
+            } else {
+                if (!qtest1(VECTOR_ELT(x, i), checker, nrules))
+                    return FALSE;
+            }
+        }
+    } else {
+        for (R_xlen_t i = 0; i < nx; i++) {
+            if (!qtest1(VECTOR_ELT(x, i), checker, nrules))
+                return FALSE;
+        }
     }
     return TRUE;
 }
 
-SEXP c_qtest(SEXP x, SEXP rules, SEXP recursive) {
+/* exported for other packages */
+Rboolean qtest(SEXP x, const char *rule) {
+    checker_t checker;
+    parse_rule(&checker, rule);
+    return qtest1(x, &checker, 1);
+}
+
+SEXP attribute_hidden c_qtest(SEXP x, SEXP rules, SEXP recursive, SEXP depth) {
     const R_len_t nrules = length(rules);
 
     if (!isString(rules))
@@ -468,7 +537,7 @@ SEXP c_qtest(SEXP x, SEXP rules, SEXP recursive) {
         parse_rule(&checker[i], CHAR(STRING_ELT(rules, i)));
     }
 
-    return LOGICAL(recursive)[0] ?
-        ScalarLogical(qtest_list(x, checker, nrules)) :
-        ScalarLogical(qtest1(x, checker, nrules));
+    if (LOGICAL(recursive)[0])
+        return ScalarLogical(qtest_list(x, checker, nrules, asCount(depth, "depth")));
+    return ScalarLogical(qtest1(x, checker, nrules));
 }
