@@ -1,10 +1,13 @@
 context("checkTibble")
 
 test_that("checkTibble", {
-  skip_if_not_installed("tibble")
-  library(tibble)
+  skip_if_not_physically_installed("tibble")
 
-  x = as_tibble(iris)
+  expect_false(testTibble(iris))
+  expect_true("tibble" %in% loadedNamespaces())
+
+  x = tibble::as_tibble(iris)
+  expect_succ_all("DataFrame", x)
   expect_succ_all("Tibble", x)
   expect_fail_all("Tibble", iris)
 
